import { DrawPile } from "./drawpile.ts";
import { AlignX, AlignY, Point, Rect, Size } from "./engine/datatypes.ts";
import { C } from "./colors.ts";
import { D } from "./engine/public.ts";

export function addButton(
  drawpile: DrawPile,
  label: string,
  rect: Rect,
  enabled: boolean,
  cbClick: () => void,
  options?: {
    endorse?: boolean;
  },
) {
  let padding = 2;
  let topLeft = rect.top;
  let topLeftPadded = topLeft.offset(new Point(padding, padding));
  let sizePadded = new Size(
    rect.size.w - padding * 2,
    rect.size.h - padding * 2,
  );
  let center = topLeft.offset(new Point(rect.size.w / 2, rect.size.h / 2));

  drawpile.addClickable(
    0,
    (hover) => {
      let [bg, fg, fgLabel] = [C.BG_UI, C.FG_TEXT, C.FG_BOLD];
      if (!enabled) {
        fgLabel = C.FG_TEXT_DISABLED;
      }

      if (enabled && options?.endorse) {
        fg = C.FG_TEXT_ENDORSED;
        fgLabel = C.FG_TEXT_ENDORSED;
      }

      if (hover) {
        [bg, fg, fgLabel] = [C.FG_BOLD, C.BG_UI, C.BG_UI];
      }
      D.fillRect(
        topLeftPadded.offset(new Point(-1, -1)),
        sizePadded.add(new Size(2, 2)),
        bg,
      );
      D.drawRect(topLeftPadded, sizePadded, fg);
      D.drawText(label, center, fgLabel, {
        alignX: AlignX.Center,
        alignY: AlignY.Middle,
      });
    },
    new Rect(topLeftPadded, sizePadded),
    enabled,
    { onClick: cbClick },
  );
}
